@@ -20,7 +20,7 @@
 #       MA 02110-1301, USA.
 
 import gtk, os, xml.dom.minidom, re, base64, mimetypes
-import cons, machines, exports, imports
+import cons, machines, exports, imports, support
 
 
 TARGET_CTD_PLAIN_TEXT = 'UTF8_STRING'
@@ -34,6 +34,47 @@ TARGETS_IMAGES = ('image/png', 'image/jpeg', 'image/bmp', 'image/tiff', 'image/x
 TARGET_WINDOWS_FILE_NAME = 'FileName'
 
 
+class Win32HtmlFormat:
+    """This class adds support for Windows "HTML Format" clipboard content type
+    Code is based on example code from http://code.activestate.com/recipes/474121/
+    written by Phillip Piper (jppx1[at]bigfoot.com)"""
+
+    MARKER_BLOCK_OUTPUT = \
+        "Version:1.0\r\n" \
+        "StartHTML:%09d\r\n" \
+        "EndHTML:%09d\r\n" \
+        "StartFragment:%09d\r\n" \
+        "EndFragment:%09d\r\n" \
+        "StartSelection:%09d\r\n" \
+        "EndSelection:%09d\r\n" \
+        "SourceURL:%s\r\n"
+
+    DEFAULT_HTML_BODY = \
+        "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 Transitional//EN\">" \
+        "<HTML><HEAD>%s</HEAD><BODY><!--StartFragment-->%s<!--EndFragment--></BODY></HTML>"
+
+    @classmethod
+    def encode(self, html_in):
+        fragment = html_in.encode(cons.STR_UTF8)
+        head = ""
+        source = cons.APP_NAME+cons.VERSION
+
+        html = self.DEFAULT_HTML_BODY % (head, fragment)
+        fragmentStart = html.index(fragment)
+        fragmentEnd = fragmentStart + len(fragment)
+
+        # How long is the prefix going to be?
+        dummyPrefix = self.MARKER_BLOCK_OUTPUT % (0, 0, 0, 0, 0, 0, source)
+        lenPrefix = len(dummyPrefix)
+
+        prefix = self.MARKER_BLOCK_OUTPUT % (
+            lenPrefix, len(html)+lenPrefix,
+            fragmentStart+lenPrefix, fragmentEnd+lenPrefix,
+            fragmentStart+lenPrefix, fragmentEnd+lenPrefix,
+            source
+        )
+        return prefix + html
+
 class ClipboardHandler:
     """Handler of Clipboard"""
 
@@ -116,9 +157,10 @@ class ClipboardHandler:
             self.dad.syntax_highlighting if not from_codebox else cons.PLAIN_TEXT_ID)
         if not from_codebox and self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
             txt_handler = exports.Export2Txt(self.dad)
-            plain_text = txt_handler.node_export_to_txt(text_buffer, "", sel_range=[iter_sel_start.get_offset(), iter_sel_end.get_offset()])
+            text_offsets_range = [iter_sel_start.get_offset(), iter_sel_end.get_offset()]
+            plain_text = txt_handler.node_export_to_txt(text_buffer, "", sel_range=text_offsets_range, check_link_target=True)
             rich_text = self.rich_text_get_from_text_buffer_selection(text_buffer, iter_sel_start, iter_sel_end)
-            targets_vector = [TARGET_CTD_PLAIN_TEXT, TARGET_CTD_RICH_TEXT, TARGETS_HTML[0]]
+            targets_vector = [TARGET_CTD_PLAIN_TEXT, TARGET_CTD_RICH_TEXT, TARGETS_HTML[0], TARGETS_HTML[1]]
             if pixbuf_target: targets_vector.append(TARGETS_IMAGES[0])
             self.clipboard.set_with_data([(t, 0, 0) for t in targets_vector],
                 self.get_func,
@@ -126,7 +168,7 @@ class ClipboardHandler:
                 (plain_text, rich_text, html_text, pixbuf_target))
         else:
             plain_text = text_buffer.get_text(iter_sel_start, iter_sel_end)
-            self.clipboard.set_with_data([(t, 0, 0) for t in (TARGET_CTD_PLAIN_TEXT, TARGETS_HTML[0])],
+            self.clipboard.set_with_data([(t, 0, 0) for t in (TARGET_CTD_PLAIN_TEXT, TARGETS_HTML[0], TARGETS_HTML[1])],
                                          self.get_func,
                                          self.clear_func,
                                          (plain_text, None, html_text, pixbuf_target))
@@ -134,9 +176,17 @@ class ClipboardHandler:
     def get_func(self, clipboard, selectiondata, info, data):
         """Connected with clipboard.set_with_data"""
         target = selectiondata.get_target()
-        if target == TARGET_CTD_PLAIN_TEXT: selectiondata.set('UTF8_STRING', 8, data[0])
+        if target == TARGET_CTD_PLAIN_TEXT: selectiondata.set(target, 8, data[0])
         elif target == TARGET_CTD_RICH_TEXT: selectiondata.set('UTF8_STRING', 8, data[1])
-        elif target == TARGETS_HTML[0]: selectiondata.set('UTF8_STRING', 8, data[2])
+        elif target in TARGETS_HTML:
+            #print target
+            if not cons.IS_WIN_OS:
+                selectiondata.set(target, 8, data[2])
+            else:
+                if target == TARGETS_HTML[0]:
+                    selectiondata.set(target, 8, data[2].encode(cons.STR_UTF16))
+                else:
+                    selectiondata.set(target, 8, Win32HtmlFormat.encode(data[2]))
         elif target == TARGET_CTD_CODEBOX:
             dom = xml.dom.minidom.Document()
             self.dad.xml_handler.codebox_element_to_xml([0, data[0], cons.TAG_PROP_LEFT], dom, dom)
@@ -166,26 +216,27 @@ class ClipboardHandler:
                     self.clipboard.request_contents(target, self.to_plain_text)
                     return
         #print targets
-        if TARGET_CTD_RICH_TEXT in targets and self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
-            self.clipboard.request_contents(TARGET_CTD_RICH_TEXT, self.to_rich_text)
-            return
-        if TARGET_CTD_CODEBOX in targets and self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
-            self.clipboard.request_contents(TARGET_CTD_CODEBOX, self.to_codebox)
-            return
-        if TARGET_CTD_TABLE in targets and self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
-            self.clipboard.request_contents(TARGET_CTD_TABLE, self.to_table, None)
-            return
-        for target in TARGETS_HTML:
-            if target in targets and self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
-                self.clipboard.request_contents(target, self.to_html)
+        if self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
+            if TARGET_CTD_RICH_TEXT in targets:
+                self.clipboard.request_contents(TARGET_CTD_RICH_TEXT, self.to_rich_text)
                 return
+            if TARGET_CTD_CODEBOX in targets:
+                self.clipboard.request_contents(TARGET_CTD_CODEBOX, self.to_codebox)
+                return
+            if TARGET_CTD_TABLE in targets:
+                self.clipboard.request_contents(TARGET_CTD_TABLE, self.to_table, None)
+                return
+            for target in TARGETS_HTML:
+                if target in targets:
+                    self.clipboard.request_contents(target, self.to_html)
+                    return
+            for target in TARGETS_IMAGES:
+                if target in targets:
+                    self.clipboard.request_contents(target, self.to_image)
+                    return
         if TARGET_URI_LIST in targets:
             self.clipboard.request_contents(TARGET_URI_LIST, self.to_uri_list)
             return
-        for target in TARGETS_IMAGES:
-            if target in targets and self.dad.syntax_highlighting == cons.RICH_TEXT_ID:
-                self.clipboard.request_contents(target, self.to_image)
-                return
         for target in TARGETS_PLAIN_TEXT:
             if target in targets:
                 self.clipboard.request_contents(target, self.to_plain_text)
@@ -244,35 +295,14 @@ class ClipboardHandler:
 
     def to_html(self, clipboard, selectiondata, data):
         """From Clipboard to HTML Text"""
-        if not cons.IS_WIN_OS and ord(selectiondata.data[0]) == 0xff and ord(selectiondata.data[1]) in [0xfe, 0xff]:
-            selection_data = selectiondata.data.decode(cons.STR_UTF16, cons.STR_IGNORE)
-        else:
-            match = re.match('.*\x00\w\x00\w\x00\w.*', selectiondata.data, re.UNICODE) # \w is alphanumeric char
-            if match: selection_data = selectiondata.data.decode(cons.STR_UTF16, cons.STR_IGNORE)
-            elif "html" in selectiondata.data or "HTML" in selectiondata.data:
-                selection_data = selectiondata.data.decode(cons.STR_UTF8, cons.STR_IGNORE)
-            else:
-                utf8_OK = False
-                utf16_OK = False
-                selection_data = ""
-                try:
-                    selection_data = selectiondata.data.decode(cons.STR_UTF16)
-                    utf16_OK = True
-                except: pass
-                try:
-                    selection_data = selectiondata.data.decode(cons.STR_UTF8)
-                    utf8_OK = True
-                except: pass
-                if utf16_OK == utf8_OK:
-                    if cons.IS_WIN_OS: selection_data = selectiondata.data.decode(cons.STR_UTF16, cons.STR_IGNORE)
-                    else: selection_data = selectiondata.data.decode(cons.STR_UTF8, cons.STR_IGNORE)
         #print "###########################"
         #print selectiondata.data
         #print "###########################"
-        #print selectiondata.data.decode(cons.STR_UTF16, cons.STR_IGNORE)
-        #print "###########################"
         #for char in selection_data: print ord(char)
+        #print "###########################"
+        selection_data = support.auto_decode_str(selectiondata.data, from_clipboard=True)
         selection_data = re.sub(cons.BAD_CHARS, "", selection_data)
+        #print selection_data
         html_import = imports.HTMLHandler(self.dad)
         xml_string = html_import.get_clipboard_selection_xml(selection_data)
         self.from_xml_string_to_buffer(xml_string)
