@@ -73,6 +73,7 @@ class CodeBoxesHandler:
         """Delete CodeBox"""
         self.dad.object_set_selection(self.curr_codebox_anchor)
         self.dad.curr_buffer.delete_selection(True, self.dad.sourceview.get_editable())
+        self.curr_codebox_anchor = None
         self.dad.sourceview.grab_focus()
 
     def dialog_codeboxhandle(self, title):
@@ -174,7 +175,7 @@ class CodeBoxesHandler:
             combobox_prog_lang.set_sensitive(radiobutton.get_active())
         def on_key_press_codeboxhandle(widget, event):
             keyname = gtk.gdk.keyval_name(event.keyval)
-            if keyname == cons.STR_RETURN:
+            if keyname == cons.STR_KEY_RETURN:
                 spinbutton_width.update()
                 spinbutton_height.update()
                 try: dialog.get_widget_for_response(gtk.RESPONSE_ACCEPT).clicked()
@@ -282,6 +283,7 @@ class CodeBoxesHandler:
         anchor.eventbox.add(anchor.frame)
         self.dad.sourceview.add_child_at_anchor(anchor.eventbox, anchor)
         anchor.eventbox.show_all()
+        self.dad.widget_set_colors(anchor.sourceview, self.dad.rt_def_fg, self.dad.rt_def_bg, True)
         for win in [gtk.TEXT_WINDOW_LEFT, gtk.TEXT_WINDOW_RIGHT, gtk.TEXT_WINDOW_TOP, gtk.TEXT_WINDOW_BOTTOM]:
             anchor.sourceview.set_border_window_size(win, 1)
         if codebox_justification:
@@ -330,10 +332,6 @@ class CodeBoxesHandler:
 
     def codebox_change_width_height(self, new_width, new_height):
         """Replace CodeBox changing Width and Height"""
-        if self.dad.user_active:
-            self.dad.user_active = False
-            user_active_restore = True
-        else: user_active_restore = False
         codebox_iter = self.dad.curr_buffer.get_iter_at_child_anchor(self.curr_codebox_anchor)
         codebox_element = [codebox_iter.get_offset(),
                            self.dad.state_machine.codebox_to_dict(self.curr_codebox_anchor, for_print=0),
@@ -345,7 +343,6 @@ class CodeBoxesHandler:
         iter_insert = self.dad.curr_buffer.get_iter_at_offset(codebox_element[0])
         self.codebox_insert(iter_insert, codebox_element[1], codebox_element[2], cursor_pos=cursor_pos)
         self.curr_codebox_anchor.sourceview.grab_focus()
-        if user_active_restore: self.dad.user_active = True
 
     def codebox_load_from_file(self, action):
         """Load the CodeBox Content From a Text File"""
@@ -393,6 +390,7 @@ class CodeBoxesHandler:
 
     def on_sourceview_event_after_codebox(self, text_view, event, anchor):
         """Called after every event on the SourceView"""
+        if not self.dad.user_active: return False
         if event.type == gtk.gdk._2BUTTON_PRESS and event.button == 1:
             support.on_sourceview_event_after_double_click_button1(self.dad, text_view, event)
         elif event.type == gtk.gdk.BUTTON_PRESS:
@@ -406,7 +404,11 @@ class CodeBoxesHandler:
         if not self.dad.user_active or self.curr_codebox_anchor != anchor: return False
         if self.dad.codebox_auto_resize and event.type == gtk.gdk.EXPOSE:
             curr_v = vscrollbar.get_value()
-            if curr_v:
+            #print curr_v+vscrollbar.get_adjustment().page_size, vscrollbar.get_adjustment().upper
+            if curr_v and ((curr_v+vscrollbar.get_adjustment().page_size) >= (vscrollbar.get_adjustment().upper-20)) and (vscrollbar.get_adjustment().page_size > curr_v):
+                # 1) the scrollbar is visible
+                # 2) we are at the scrolling end
+                # 3) the scrolling area is little, this step will probably cause the scrollbar to hide again
                 self.curr_v = curr_v
                 if not self.dad.codebox_sentinel_id: self.dad.codebox_sentinel_start()
         return False
@@ -416,13 +418,18 @@ class CodeBoxesHandler:
         if not self.dad.user_active or self.curr_codebox_anchor != anchor: return False
         if self.dad.codebox_auto_resize and event.type == gtk.gdk.EXPOSE:
             curr_h = hscrollbar.get_value()
-            if curr_h:
+            #print curr_h+hscrollbar.get_adjustment().page_size, hscrollbar.get_adjustment().upper
+            if curr_h and ((curr_h+hscrollbar.get_adjustment().page_size) >= (hscrollbar.get_adjustment().upper-20)) and (hscrollbar.get_adjustment().page_size > curr_h):
+                # 1) the scrollbar is visible
+                # 2) we are at the scrolling end
+                # 3) the scrolling area is little, this step will probably cause the scrollbar to hide again
                 self.curr_h = curr_h
                 if not self.dad.codebox_sentinel_id: self.dad.codebox_sentinel_start()
         return False
 
     def on_key_press_sourceview_codebox(self, widget, event, anchor):
         """Extend the Default Right-Click Menu of the CodeBox"""
+        if not self.dad.user_active: return False
         if event.state & gtk.gdk.CONTROL_MASK:
             keyname = gtk.gdk.keyval_name(event.keyval)
             self.curr_codebox_anchor = anchor
@@ -438,6 +445,7 @@ class CodeBoxesHandler:
 
     def on_mouse_button_clicked_codebox(self, widget, event, anchor):
         """Catches mouse buttons clicks"""
+        if not self.dad.user_active: return False
         self.curr_codebox_anchor = anchor
         if event.button != 3:
             self.dad.object_set_selection(self.curr_codebox_anchor)
@@ -445,10 +453,12 @@ class CodeBoxesHandler:
 
     def on_sourceview_populate_popup_codebox(self, textview, menu, anchor):
         """Extend the Default Right-Click Menu of the CodeBox"""
+        if not self.dad.user_active: return
         self.dad.menu_populate_popup(menu, cons.get_popup_menu_entries_codebox(self), self.dad.orphan_accel_group)
 
     def on_sourceview_motion_notify_event_codebox(self, text_view, event):
         """Update the cursor image if the pointer moved"""
+        if not self.dad.user_active: return False
         text_view.set_editable(not self.dad.is_curr_node_read_only())
         x, y = text_view.window_to_buffer_coords(gtk.TEXT_WINDOW_TEXT, int(event.x), int(event.y))
         support.sourceview_cursor_and_tooltips_handler(self.dad, text_view, x, y)
