@@ -64,7 +64,7 @@ class FindReplace:
             button.set_image(gtk.image_new_from_stock(gtk.STOCK_UNDO, gtk.ICON_SIZE_BUTTON))
         except: pass
         def on_key_press_iterated_find_dialog(widget, event):
-            if gtk.gdk.keyval_name(event.keyval) == cons.STR_RETURN:
+            if gtk.gdk.keyval_name(event.keyval) == cons.STR_KEY_RETURN:
                 try: dialog.get_widget_for_response(1).clicked()
                 except: print cons.STR_PYGTK_222_REQUIRED
                 return True
@@ -640,10 +640,8 @@ class FindReplace:
             self.liststore.remove(list_iter)
             return
         self.dad.treeview_safe_set_cursor(tree_iter)
-        if model[list_iter][1] != 0 and model[list_iter][2] != 0:
-            self.dad.curr_buffer.move_mark(self.dad.curr_buffer.get_insert(),
-                                           self.dad.curr_buffer.get_iter_at_offset(model[list_iter][1]))
-            self.dad.curr_buffer.move_mark(self.dad.curr_buffer.get_selection_bound(),
-                                           self.dad.curr_buffer.get_iter_at_offset(model[list_iter][2]))
-            self.dad.sourceview.scroll_to_mark(self.dad.curr_buffer.get_insert(), 0.25)
-        else: self.dad.sourceview.grab_focus()
+        self.dad.curr_buffer.move_mark(self.dad.curr_buffer.get_insert(),
+                                       self.dad.curr_buffer.get_iter_at_offset(model[list_iter][1]))
+        self.dad.curr_buffer.move_mark(self.dad.curr_buffer.get_selection_bound(),
+                                       self.dad.curr_buffer.get_iter_at_offset(model[list_iter][2]))
+        self.dad.sourceview.scroll_to_mark(self.dad.curr_buffer.get_insert(), 0.25)
