@@ -31,8 +31,8 @@ class ListsHandler:
         """Lists Handler boot"""
         self.dad = dad
 
-    def list_todo_handler(self, text_buffer=None):
-        """Handler of the ToDo List"""
+    def list_handler(self, target_list_num_id, text_buffer=None):
+        """Unified Handler of Lists"""
         if not text_buffer: text_buffer = self.dad.curr_buffer
         if text_buffer.get_has_selection():
             iter_start, sel_end = text_buffer.get_selection_bounds()
@@ -42,205 +42,63 @@ class ListsHandler:
             iter_start = text_buffer.get_iter_at_mark(text_buffer.get_insert())
         # get info about the paragraph starting with iter_start ([Number, Whether multiple line, List Start Iter Offset])
         list_info = self.get_paragraph_list_info(iter_start)
-        if list_info[0] == None:
-            # this is not a list
-            first_iteration = True
-            while first_iteration or new_par_offset < end_offset:
-                first_iteration = False
-                iter_start, iter_end = self.get_paragraph_iters(text_buffer=text_buffer, force_iter=iter_start)
-                if not iter_start:
-                    # it's an empty paragraph
-                    iter_start = text_buffer.get_iter_at_mark(text_buffer.get_insert())
-                    text_buffer.insert(iter_start, cons.CHAR_LISTTODO + cons.CHAR_SPACE)
-                    break
-                elif self.is_list_indented_continuation(iter_start):
-                    new_par_offset = iter_end.get_offset() + 1
-                else:
-                    new_par_offset = iter_end.get_offset() + 2 + 1
-                    text_buffer.insert(iter_start, cons.CHAR_LISTTODO + cons.CHAR_SPACE)
-                    end_offset += 2
-                iter_start = text_buffer.get_iter_at_offset(new_par_offset)
-                if not iter_start: break
-        elif list_info[0] == 0:
-            # this is a bulleted list paragraph and we turn it into todo list
-            self.list_adjust_ahead(None, list_info[2]-1, "bul2tod", text_buffer)
-        elif list_info[0] == -1:
-            # this is a todo list and we turn it into normal text
-            first_iteration = True
-            while first_iteration or new_par_offset < end_offset:
-                first_iteration = False
-                iter_start, iter_end = self.get_paragraph_iters(text_buffer=text_buffer, force_iter=iter_start)
-                if not iter_start: break
-                if self.is_list_indented_continuation(iter_start):
-                    new_par_offset = iter_end.get_offset() + 1
-                else:
-                    new_par_offset = iter_end.get_offset() - 2 + 1
-                    iter_end_deletion = iter_start.copy()
-                    iter_end_deletion.forward_chars(2)
-                    text_buffer.delete(iter_start, iter_end_deletion)
-                    end_offset -= 2
-                iter_start = text_buffer.get_iter_at_offset(new_par_offset)
-                if not iter_start: break
-        else:
-            # this is a numbered list and we turn it into todo list
-            self.list_adjust_ahead(None, list_info[2]-1, "num2tod", text_buffer)
+        leading_num_count = 0
+        while leading_num_count == 0 or new_par_offset < end_offset:
+            leading_num_count += 1
+            iter_start, iter_end = self.get_paragraph_iters(text_buffer=text_buffer, force_iter=iter_start)
+            if not iter_start:
+                if leading_num_count == 1:
+                    # this is the first iteration
+                    if list_info[0] != target_list_num_id:
+                        # the target list type differs from this paragraph list type
+                        iter_start = text_buffer.get_iter_at_mark(text_buffer.get_insert())
+                        if target_list_num_id == -1: text_buffer.insert(iter_start, cons.CHAR_LISTTODO + cons.CHAR_SPACE)
+                        elif target_list_num_id == 0: text_buffer.insert(iter_start, cons.CHAR_LISTBUL + cons.CHAR_SPACE)
+                        else: text_buffer.insert(iter_start, "1. ")
+                break
+            if self.is_list_indented_continuation(iter_start):
+                new_par_offset = iter_end.get_offset()
+                leading_num_count -= 1
+            else:
+                iter_start, iter_end, chars_rm = self.list_check_n_remove_old_list_type_leading(iter_start, iter_end, text_buffer)
+                end_offset -= chars_rm
+                if list_info[0] != target_list_num_id:
+                    # the target list type differs from this paragraph list type
+                    if target_list_num_id == -1:
+                        new_par_offset = iter_end.get_offset() + 2
+                        end_offset += 2
+                        text_buffer.insert(iter_start, cons.CHAR_LISTTODO + cons.CHAR_SPACE)
+                    elif target_list_num_id == 0:
+                        new_par_offset = iter_end.get_offset() + 2
+                        end_offset += 2
+                        text_buffer.insert(iter_start, cons.CHAR_LISTBUL + cons.CHAR_SPACE)
+                    else:
+                        leading_str = "%s. " % leading_num_count
+                        new_par_offset = iter_end.get_offset() + len(leading_str)
+                        end_offset += len(leading_str)
+                        text_buffer.insert(iter_start, leading_str)
+                else: new_par_offset = iter_end.get_offset()
+            iter_start = text_buffer.get_iter_at_offset(new_par_offset+1)
+            if not iter_start: break
 
-    def list_bulleted_handler(self, text_buffer=None):
-        """Handler of the Bulleted List"""
-        if not text_buffer: text_buffer = self.dad.curr_buffer
-        if text_buffer.get_has_selection():
-            iter_start, sel_end = text_buffer.get_selection_bounds()
-            end_offset = sel_end.get_offset() - 1
-        else:
-            end_offset = 0
-            iter_start = text_buffer.get_iter_at_mark(text_buffer.get_insert())
-        # get info about the paragraph starting with iter_start ([Number, Whether multiple line, List Start Iter Offset])
+    def list_check_n_remove_old_list_type_leading(self, iter_start, iter_end, text_buffer):
+        """Clean List Start Characters"""
+        start_offset = iter_start.get_offset()
+        end_offset = iter_end.get_offset()
         list_info = self.get_paragraph_list_info(iter_start)
-        if list_info[0] == None:
-            # this is not a list
-            first_iteration = True
-            while first_iteration or new_par_offset < end_offset:
-                first_iteration = False
-                iter_start, iter_end = self.get_paragraph_iters(text_buffer=text_buffer, force_iter=iter_start)
-                if not iter_start:
-                    # it's an empty paragraph
-                    iter_start = text_buffer.get_iter_at_mark(text_buffer.get_insert())
-                    text_buffer.insert(iter_start, cons.CHAR_LISTBUL + cons.CHAR_SPACE)
-                    break
-                elif self.is_list_indented_continuation(iter_start):
-                    new_par_offset = iter_end.get_offset() + 1
-                else:
-                    new_par_offset = iter_end.get_offset() + 2 + 1
-                    text_buffer.insert(iter_start, cons.CHAR_LISTBUL + cons.CHAR_SPACE)
-                    end_offset += 2
-                iter_start = text_buffer.get_iter_at_offset(new_par_offset)
-                if not iter_start: break
-        elif list_info[0] == 0:
-            # this is a bulleted list paragraph and we turn it into normal text
-            first_iteration = True
-            while first_iteration or new_par_offset < end_offset:
-                first_iteration = False
-                iter_start, iter_end = self.get_paragraph_iters(text_buffer=text_buffer, force_iter=iter_start)
-                if not iter_start: break
-                if self.is_list_indented_continuation(iter_start):
-                    new_par_offset = iter_end.get_offset() + 1
-                else:
-                    new_par_offset = iter_end.get_offset() -2 + 1
-                    iter_end_deletion = iter_start.copy()
-                    iter_end_deletion.forward_chars(2)
-                    text_buffer.delete(iter_start, iter_end_deletion)
-                    end_offset -= 2
-                iter_start = text_buffer.get_iter_at_offset(new_par_offset)
-                if not iter_start: break
-        elif list_info[0] == -1:
-            # this is a todo list and we turn it into bulleted list
-            self.list_adjust_ahead(None, list_info[2]-1, "tod2bul", text_buffer)
-        else:
-            # this is a numbered list and we turn it into bulleted list
-            self.list_adjust_ahead(None, list_info[2]-1, "num2bul", text_buffer)
-
-    def list_numbered_handler(self, text_buffer=None):
-        """Handler of the Numbered List"""
-        if not text_buffer: text_buffer = self.dad.curr_buffer
-        if text_buffer.get_has_selection():
-            iter_start, sel_end = text_buffer.get_selection_bounds()
-            end_offset = sel_end.get_offset() - 1
-        else:
-            end_offset = 0
-            iter_start = text_buffer.get_iter_at_mark(text_buffer.get_insert())
-        # get info about the paragraph starting with iter_start ([Number, Whether multiple line, List Start Iter Offset])
-        list_info = self.get_paragraph_list_info(iter_start)
-        if list_info[0] == None:
-            # this is not a list
-            leading_num_count = 0
-            while leading_num_count == 0 or new_par_offset < end_offset:
-                leading_num_count += 1
-                iter_start, iter_end = self.get_paragraph_iters(text_buffer=text_buffer, force_iter=iter_start)
-                if not iter_start:
-                    # it's an empty paragraph
-                    iter_start = text_buffer.get_iter_at_mark(text_buffer.get_insert())
-                    text_buffer.insert(iter_start, "1. ")
-                    break
-                elif self.is_list_indented_continuation(iter_start):
-                    new_par_offset = iter_end.get_offset() + 1
-                else:
-                    leading_str = "%s. " % leading_num_count
-                    new_par_offset = iter_end.get_offset() + len(leading_str) + 1
-                    text_buffer.insert(iter_start, leading_str)
-                    end_offset += len(leading_str)
-                iter_start = text_buffer.get_iter_at_offset(new_par_offset)
-                if not iter_start: break
-        elif list_info[0] == 0:
-            # this is a bulleted list and we turn it into numbered list
-            self.list_adjust_ahead(0, list_info[2]-1, "bul2num", text_buffer)
-        elif list_info[0] == -1:
-            # this is a todo list and we turn it into numbered list
-            self.list_adjust_ahead(0, list_info[2]-1, "tod2num", text_buffer)
-        else:
-            # this is a numbered list paragraph and we turn it into normal text
-            first_iteration = True
-            while first_iteration or new_par_offset < end_offset:
-                first_iteration = False
-                iter_start, iter_end = self.get_paragraph_iters(text_buffer=text_buffer, force_iter=iter_start)
-                if not iter_start: break
-                leading_str = "%s. " % list_info[0]
-                if self.is_list_indented_continuation(iter_start):
-                    new_par_offset = iter_end.get_offset() + 1
-                else:
-                    new_par_offset = iter_end.get_offset() -len(leading_str) + 1
-                    iter_end_deletion = iter_start.copy()
-                    iter_end_deletion.forward_chars(len(leading_str))
-                    text_buffer.delete(iter_start, iter_end_deletion)
-                    end_offset -= len(leading_str)
-                iter_start = text_buffer.get_iter_at_offset(new_par_offset)
-                if not iter_start: break
-
-    def list_adjust_ahead(self, curr_num, offset_start, adj_type, text_buffer):
-        """Adjust the Following Numbering"""
-        if offset_start >= 0:
-            iter_start = text_buffer.get_iter_at_offset(offset_start)
-            while iter_start.get_char() != cons.CHAR_NEWLINE:
-                if not iter_start.forward_char(): return # end of buffer
-            if not iter_start.forward_char(): return
-        else: iter_start = text_buffer.get_iter_at_offset(0)
-        # we're at the beginning of a list item or subsequent indented line of a list item
-        if iter_start.get_char() == cons.CHAR_SPACE:
-            if iter_start.forward_char() and iter_start.get_char() == cons.CHAR_SPACE\
-            and iter_start.forward_char() and iter_start.get_char() == cons.CHAR_SPACE:
-                self.list_adjust_ahead(curr_num, iter_start.get_offset(), adj_type, text_buffer) # go on searching
-            else: return # not an indentation
-        elif adj_type[0:3] == "num":
-            match = re.match('[1-9]', iter_start.get_char())
-            if not match: return
+        if list_info[0] != None:
+            leading_chars_num = self.get_leading_chars_num_from_list_info_num(list_info[0])
             iter_end = iter_start.copy()
-            while iter_end.forward_char() and re.match('[0-9]', iter_end.get_char()): pass
-            if iter_end.get_char() != ".": return # something's wrong
-            self.list_adjust_ahead_write_in(iter_start, iter_end, curr_num, adj_type, text_buffer)
-        elif adj_type[0:3] == "bul":
-            if iter_start.get_char() != cons.CHAR_LISTBUL: return
-            iter_end = iter_start.copy()
-            self.list_adjust_ahead_write_in(iter_start, iter_end, curr_num, adj_type, text_buffer)
-        elif adj_type[0:3] == "tod":
-            if not iter_start.get_char() in [cons.CHAR_LISTTODO, cons.CHAR_LISTDONEOK, cons.CHAR_LISTDONEFAIL]: return
-            iter_end = iter_start.copy()
-            self.list_adjust_ahead_write_in(iter_start, iter_end, curr_num, adj_type, text_buffer)
-        else: print "bad adj_type", adj_type
+            iter_end.forward_chars(leading_chars_num)
+            text_buffer.delete(iter_start, iter_end)
+            end_offset -= leading_chars_num
+        else: leading_chars_num = 0
+        return text_buffer.get_iter_at_offset(start_offset), text_buffer.get_iter_at_offset(end_offset), leading_chars_num
 
-    def list_adjust_ahead_write_in(self, iter_start, iter_end, curr_num, adj_type, text_buffer):
-        """Write a Replacement of List Point"""
-        iter_end.forward_char()
-        text_buffer.delete(iter_start, iter_end)
-        if adj_type[-3:] == "num":
-            curr_num += 1
-            text_buffer.insert(iter_start, '%s.' % curr_num)
-            self.list_adjust_ahead(curr_num, iter_start.get_offset(), adj_type, text_buffer)
-        elif adj_type[-3:] == "bul":
-            text_buffer.insert(iter_start, cons.CHAR_LISTBUL)
-            self.list_adjust_ahead(None, iter_start.get_offset(), adj_type, text_buffer)
-        elif adj_type[-3:] == "tod":
-            text_buffer.insert(iter_start, cons.CHAR_LISTTODO)
-            self.list_adjust_ahead(None, iter_start.get_offset(), adj_type, text_buffer)
-        else: print "bad adj_type", adj_type
+    def get_leading_chars_num_from_list_info_num(self, list_info_num):
+        """From List Info to List Prefix"""
+        if list_info_num > 0: return len("%s. " % list_info_num)
+        return 2
 
     def list_get_number(self, iter_first_paragraph):
         """Returns a Number or None (0 fot the bulleted list)"""
