@@ -19,10 +19,30 @@
 #       Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 #       MA 02110-1301, USA.
 
-import gtk, pango, os, webbrowser, re
+import gtk, pango, locale, os, webbrowser, re
 import cons, config
 
 
+def auto_decode_str(in_str, from_clipboard=False):
+    """Try to Detect Encoding and Decode"""
+    if in_str.startswith("\xEF\xBB\xBF"): # UTF-8 "BOM"
+        encodings = ["utf-8-sig"]
+    elif in_str.startswith(("\xFF\xFE", "\xFE\xFF")): # UTF-16 BOMs
+        encodings = [cons.STR_UTF16]
+    elif from_clipboard:
+        encodings = [cons.STR_UTF8, cons.STR_UTF16]
+    else:
+        encodings = [cons.STR_UTF8, cons.STR_ISO_8859, cons.STR_UTF16, "utf-16le", locale.getdefaultlocale()[1]]
+    for enc in encodings:
+        try:
+            out_str = in_str.decode(enc)
+            #print enc
+            break
+        except: pass
+    else:
+        out_str = unicode(in_str, cons.STR_UTF8, cons.STR_IGNORE)
+    return out_str
+
 def apply_tag_try_automatic_bounds(dad, text_buffer=None, iter_start=None):
     """Try to Select a Word Forward/Backward the Cursor"""
     if not text_buffer: text_buffer = dad.curr_buffer
@@ -31,7 +51,13 @@ def apply_tag_try_automatic_bounds(dad, text_buffer=None, iter_start=None):
     curr_char = iter_end.get_char()
     # 1) select alphanumeric + special
     match = re.match('\w', curr_char, re.UNICODE)
-    if not match and not curr_char in dad.selword_chars: return False
+    if not match and not curr_char in dad.selword_chars:
+        iter_start.backward_char()
+        iter_end.backward_char()
+        curr_char = iter_end.get_char()
+        match = re.match('\w', curr_char, re.UNICODE)
+        if not match and not curr_char in dad.selword_chars:
+            return False
     while match or curr_char in dad.selword_chars:
         if not iter_end.forward_char(): break # end of buffer
         curr_char = iter_end.get_char()
@@ -95,9 +121,9 @@ def on_sourceview_event_after_key_press(dad, text_view, event):
     text_buffer = text_view.get_buffer()
     keyname = gtk.gdk.keyval_name(event.keyval)
     if (event.state & gtk.gdk.SHIFT_MASK):
-        if keyname == cons.STR_RETURN:
+        if keyname == cons.STR_KEY_RETURN:
             text_buffer.insert(text_buffer.get_iter_at_mark(text_buffer.get_insert()), 3*cons.CHAR_SPACE)
-    elif keyname == cons.STR_RETURN:
+    elif keyname == cons.STR_KEY_RETURN:
         iter_insert = text_buffer.get_iter_at_mark(text_buffer.get_insert())
         if not iter_insert:
             return False
@@ -133,10 +159,7 @@ def on_sourceview_event_after_key_press(dad, text_view, event):
             return False # former was an empty paragraph => list quit
         if list_info[0] == 0: text_buffer.insert(iter_insert, cons.CHAR_LISTBUL + cons.CHAR_SPACE)
         elif list_info[0] == -1: text_buffer.insert(iter_insert, cons.CHAR_LISTTODO + cons.CHAR_SPACE)
-        else:
-            curr_num = list_info[0] + 1
-            text_buffer.insert(iter_insert, '%s. ' % curr_num)
-            dad.lists_handler.list_adjust_ahead(curr_num, iter_insert.get_offset(), "num2num", text_buffer)
+        else: text_buffer.insert(iter_insert, '%s. ' % (list_info[0] + 1))
     elif keyname == "space":
         iter_insert = text_buffer.get_iter_at_mark(text_buffer.get_insert())
         if iter_insert == None: return False
@@ -166,20 +189,19 @@ def on_sourceview_event_after_key_press(dad, text_view, event):
             # Start bulleted list on "* " at line start
             elif iter_start.get_char() == cons.CHAR_STAR and iter_start.get_line_offset() == 0:
                 text_buffer.delete(iter_start, iter_insert)
-                dad.lists_handler.list_bulleted_handler(text_buffer=text_buffer)
-            # Start todo list on "[ ]" at line start
+                dad.lists_handler.list_handler(0, text_buffer=text_buffer)
+            # Start todo list on "[]" at line start
             elif iter_start.get_char() == cons.CHAR_SQ_BR_CLOSE and iter_start.backward_char()\
-            and iter_start.get_char() == cons.CHAR_SPACE and iter_start.backward_char()\
             and iter_start.get_char() == cons.CHAR_SQ_BR_OPEN\
             and iter_start.get_line_offset() == 0:
                 text_buffer.delete(iter_start, iter_insert)
-                dad.lists_handler.list_todo_handler(text_buffer=text_buffer)
+                dad.lists_handler.list_handler(-1, text_buffer=text_buffer)
     return False
 
 def sourceview_cursor_and_tooltips_handler(dad, text_view, x, y):
     """Looks at all tags covering the position (x, y) in the text view,
        and if one of them is a link, change the cursor to the HAND2 cursor"""
-    hovering_link = False
+    hovering_link_iter_offset = -1
     text_iter = text_view.get_iter_at_location(x, y)
     if dad.lists_handler.is_list_todo_beginning(text_iter):
         text_view.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(gtk.gdk.Cursor(gtk.gdk.X_CURSOR))
@@ -190,7 +212,7 @@ def sourceview_cursor_and_tooltips_handler(dad, text_view, x, y):
     for tag in tags:
         tag_name = tag.get_property("name")
         if tag_name and tag_name[0:4] == cons.TAG_LINK:
-            hovering_link = True
+            hovering_link_iter_offset = text_iter.get_offset()
             tooltip = dad.sourceview_hovering_link_get_tooltip(tag_name[5:])
             break
     else:
@@ -201,13 +223,13 @@ def sourceview_cursor_and_tooltips_handler(dad, text_view, x, y):
             if anchor and "pixbuf" in dir(anchor):
                 pixbuf_attrs = dir(anchor.pixbuf)
                 if "link" in pixbuf_attrs and anchor.pixbuf.link:
-                    hovering_link = True
+                    hovering_link_iter_offset = text_iter.get_offset()
                     tooltip = dad.sourceview_hovering_link_get_tooltip(anchor.pixbuf.link)
                     break
-    if hovering_link != dad.hovering_over_link:
-        dad.hovering_over_link = hovering_link
-        #print "link", dad.hovering_over_link
-    if dad.hovering_over_link:
+    if dad.hovering_link_iter_offset != hovering_link_iter_offset:
+        dad.hovering_link_iter_offset = hovering_link_iter_offset
+        #print "link", dad.hovering_link_iter_offset
+    if dad.hovering_link_iter_offset >= 0:
         text_view.get_window(gtk.TEXT_WINDOW_TEXT).set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND2))
         text_view.set_tooltip_text(tooltip)
     else:
@@ -223,6 +245,15 @@ def rich_text_node_modify_codeboxes_font(start_iter, code_font):
             anchor.sourceview.modify_font(pango.FontDescription(code_font))
         if not curr_iter.forward_char(): break
 
+def rich_text_node_modify_codeboxes_color(start_iter, dad):
+    """Modify Color to CodeBoxes"""
+    curr_iter = start_iter.copy()
+    while 1:
+        anchor = curr_iter.get_child_anchor()
+        if anchor and "sourcebuffer" in dir(anchor):
+            dad.widget_set_colors(anchor.sourceview, dad.rt_def_fg, dad.rt_def_bg, True)
+        if not curr_iter.forward_char(): break
+
 def text_file_rm_emptylines(filepath):
     """Remove empty lines in a text file"""
     overwrite_needed = False
@@ -241,19 +272,14 @@ def text_file_rm_emptylines(filepath):
 
 def get_proper_platform_filepath(filepath_in, is_file):
     """From Slash to Backslash when needed"""
-    filepath_out = ""
+    filepath_out = filepath_in
     if cons.IS_WIN_OS:
         if cons.CHAR_SLASH in filepath_in:
             filepath_out = filepath_in.replace(cons.CHAR_SLASH, cons.CHAR_BSLASH)
     else:
         if cons.CHAR_BSLASH in filepath_in:
             filepath_out = filepath_in.replace(cons.CHAR_BSLASH, cons.CHAR_SLASH)
-    if filepath_out:
-        if is_file:
-            if os.path.isfile(filepath_out): return filepath_out
-        else:
-            if os.path.isdir(filepath_out): return filepath_out
-    return filepath_in
+    return filepath_out
 
 def clean_from_chars_not_for_filename(filename_in):
     """Clean a string from chars not good for filename"""
@@ -309,13 +335,16 @@ def strip_trailing_spaces(text_buffer):
                 break
     return cleaned_lines
 
-def get_next_chars_from_iter_are(iter_start, num, chars):
+def get_next_chars_from_iter_are(iter_start, num_n_chars):
     """Returns True if the Given Chars are the next 'num' after iter"""
-    text_iter = iter_start.copy()
-    for i in range(num):
-        if text_iter.get_char() != chars[i]: return False
-        if i != num-1 and not text_iter.forward_char(): return False
-    return True
+    for num_n_chars_elem in num_n_chars:
+        text_iter = iter_start.copy()
+        (num, chars) = num_n_chars_elem
+        for i in range(num):
+            if text_iter.get_char().encode(cons.STR_UTF8) != chars[i]: break
+            if i != num-1 and not text_iter.forward_char(): break
+        else: return True
+    return False
 
 def get_former_line_indentation(iter_start):
     """Returns the indentation of the former paragraph or empty string"""
@@ -587,7 +616,7 @@ def dialog_choose_element_in_list(father_win, title, elements_list, column_title
         model, list_parms.sel_iter = elements_treeviewselection.get_selected()
     def on_key_press_elementslistdialog(widget, event):
         keyname = gtk.gdk.keyval_name(event.keyval)
-        if keyname == cons.STR_RETURN:
+        if keyname == cons.STR_KEY_RETURN:
             try: dialog.get_widget_for_response(gtk.RESPONSE_ACCEPT).clicked()
             except: print cons.STR_PYGTK_222_REQUIRED
             return True
@@ -623,7 +652,7 @@ def dialog_img_n_entry(father_win, title, entry_content, img_stock):
     content_area.pack_start(hbox)
     def on_key_press_anchoreditdialog(widget, event):
         keyname = gtk.gdk.keyval_name(event.keyval)
-        if keyname == cons.STR_RETURN:
+        if keyname == cons.STR_KEY_RETURN:
             try: dialog.get_widget_for_response(gtk.RESPONSE_ACCEPT).clicked()
             except: print cons.STR_PYGTK_222_REQUIRED
             return True
@@ -721,7 +750,7 @@ def dialog_image_handle(father_win, title, original_pixbuf):
         image_load_into_dialog()
     def on_key_press_imagehandledialog(widget, event):
         keyname = gtk.gdk.keyval_name(event.keyval)
-        if keyname == cons.STR_RETURN:
+        if keyname == cons.STR_KEY_RETURN:
             spinbutton_width.update()
             spinbutton_height.update()
             try: dialog.get_widget_for_response(gtk.RESPONSE_ACCEPT).clicked()
@@ -763,7 +792,7 @@ def dialog_node_delete(father_win, warning_label):
     content_area.pack_start(hbox)
     def on_key_press_nodedeletedialog(widget, event):
         keyname = gtk.gdk.keyval_name(event.keyval)
-        if keyname == cons.STR_RETURN:
+        if keyname == cons.STR_KEY_RETURN:
             try: dialog.get_widget_for_response(gtk.RESPONSE_ACCEPT).clicked()
             except: print cons.STR_PYGTK_222_REQUIRED
             return True
@@ -804,7 +833,7 @@ def dialog_exit_del_temp_files(dad):
     content_area.pack_start(hbox)
     def on_key_press_exitdialog(widget, event):
         keyname = gtk.gdk.keyval_name(event.keyval)
-        if keyname == cons.STR_RETURN:
+        if keyname == cons.STR_KEY_RETURN:
             try: dialog.get_widget_for_response(1).clicked()
             except: print cons.STR_PYGTK_222_REQUIRED
             return True
@@ -850,7 +879,7 @@ def dialog_exit_save(father_win):
     content_area.pack_start(hbox)
     def on_key_press_exitdialog(widget, event):
         keyname = gtk.gdk.keyval_name(event.keyval)
-        if keyname == cons.STR_RETURN:
+        if keyname == cons.STR_KEY_RETURN:
             try: dialog.get_widget_for_response(2).clicked()
             except: print cons.STR_PYGTK_222_REQUIRED
             return True
@@ -870,6 +899,7 @@ def dialog_link_handle(dad, title, sel_tree_iter):
     class LinksParms:
         def __init__(self):
             self.sel_iter = sel_tree_iter if sel_tree_iter else dad.links_entries['node']
+            self.first_in = True
     links_parms = LinksParms()
     dialog = gtk.Dialog(title=title,
         parent=dad.window,
@@ -878,7 +908,7 @@ def dialog_link_handle(dad, title, sel_tree_iter):
         gtk.STOCK_OK, gtk.RESPONSE_ACCEPT) )
     dialog.set_position(gtk.WIN_POS_CENTER_ON_PARENT)
     dialog.set_default_size(600, 500)
-    
+
     hbox_webs = gtk.HBox()
     image_webs = gtk.Image()
     image_webs.set_from_stock("link_website", gtk.ICON_SIZE_BUTTON)
@@ -889,7 +919,7 @@ def dialog_link_handle(dad, title, sel_tree_iter):
     hbox_webs.pack_start(radiobutton_webs, expand=False)
     hbox_webs.pack_start(entry_webs)
     hbox_webs.set_spacing(5)
-    
+
     hbox_file = gtk.HBox()
     image_file = gtk.Image()
     image_file.set_from_stock(gtk.STOCK_FILE, gtk.ICON_SIZE_BUTTON)
@@ -904,7 +934,7 @@ def dialog_link_handle(dad, title, sel_tree_iter):
     hbox_file.pack_start(entry_file)
     hbox_file.pack_start(button_browse_file, expand=False)
     hbox_file.set_spacing(5)
-    
+
     hbox_folder = gtk.HBox()
     image_folder = gtk.Image()
     image_folder.set_from_stock(gtk.STOCK_DIRECTORY, gtk.ICON_SIZE_BUTTON)
@@ -919,7 +949,7 @@ def dialog_link_handle(dad, title, sel_tree_iter):
     hbox_folder.pack_start(entry_folder)
     hbox_folder.pack_start(button_browse_folder, expand=False)
     hbox_folder.set_spacing(5)
-    
+
     hbox_node = gtk.HBox()
     image_node = gtk.Image()
     image_node.set_from_stock("cherrytree", gtk.ICON_SIZE_BUTTON)
@@ -928,11 +958,12 @@ def dialog_link_handle(dad, title, sel_tree_iter):
     hbox_node.pack_start(image_node, expand=False)
     hbox_node.pack_start(radiobutton_node)
     hbox_node.set_spacing(5)
-    
+
     hbox_detail = gtk.HBox()
-    
+
     treeview_2 = gtk.TreeView(dad.treestore)
     treeview_2.set_headers_visible(False)
+    treeview_2.set_search_column(1)
     renderer_pixbuf_2 = gtk.CellRendererPixbuf()
     renderer_text_2 = gtk.CellRendererText()
     column_2 = gtk.TreeViewColumn()
@@ -945,18 +976,11 @@ def dialog_link_handle(dad, title, sel_tree_iter):
     scrolledwindow = gtk.ScrolledWindow()
     scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
     scrolledwindow.add(treeview_2)
-    config.get_tree_expanded_collapsed_string(dad)
-    config.set_tree_expanded_collapsed_string(dad, treeview=treeview_2)
-    if links_parms.sel_iter:
-        sel_path = dad.treestore.get_path(links_parms.sel_iter)
-        treeview_2.expand_to_path(sel_path)
-        treeview_2.set_cursor(sel_path)
-        treeview_2.scroll_to_cell(sel_path)
-    
+
     vbox_anchor = gtk.VBox()
     label_over = gtk.Label()
     label_below = gtk.Label()
-    
+
     hbox_anchor = gtk.HBox()
     entry_anchor = gtk.Entry()
     entry_anchor.set_text(dad.links_entries['anch'])
@@ -964,19 +988,19 @@ def dialog_link_handle(dad, title, sel_tree_iter):
     button_browse_anchor.set_image(gtk.image_new_from_stock("anchor", gtk.ICON_SIZE_BUTTON))
     hbox_anchor.pack_start(entry_anchor)
     hbox_anchor.pack_start(button_browse_anchor, expand=False)
-    
+
     frame_anchor = gtk.Frame(label="<b>"+_("Anchor Name (optional)")+"</b>")
     frame_anchor.get_label_widget().set_use_markup(True)
     frame_anchor.set_shadow_type(gtk.SHADOW_NONE)
     frame_anchor.add(hbox_anchor)
-    
+
     vbox_anchor.pack_start(label_over)
     vbox_anchor.pack_start(frame_anchor, expand=False)
     vbox_anchor.pack_start(label_below)
-    
+
     hbox_detail.pack_start(scrolledwindow)
     hbox_detail.pack_start(vbox_anchor, expand=False)
-    
+
     content_area = dialog.get_content_area()
     content_area.pack_start(hbox_webs, expand=False)
     content_area.pack_start(hbox_file, expand=False)
@@ -984,17 +1008,31 @@ def dialog_link_handle(dad, title, sel_tree_iter):
     content_area.pack_start(hbox_node, expand=False)
     content_area.pack_start(hbox_detail)
     content_area.set_spacing(5)
-    
+
     radiobutton_webs.set_active(dad.link_type == cons.LINK_TYPE_WEBS)
     radiobutton_node.set_active(dad.link_type == cons.LINK_TYPE_NODE)
     radiobutton_file.set_active(dad.link_type == cons.LINK_TYPE_FILE)
     radiobutton_folder.set_active(dad.link_type == cons.LINK_TYPE_FOLD)
-    
+
     def link_type_changed_on_dialog():
         entry_webs.set_sensitive(dad.link_type == cons.LINK_TYPE_WEBS)
         hbox_detail.set_sensitive(dad.link_type == cons.LINK_TYPE_NODE)
         entry_file.set_sensitive(dad.link_type == cons.LINK_TYPE_FILE)
         entry_folder.set_sensitive(dad.link_type == cons.LINK_TYPE_FOLD)
+        if dad.link_type == cons.LINK_TYPE_WEBS: entry_webs.grab_focus()
+        elif dad.link_type == cons.LINK_TYPE_NODE:
+            treeview_2.grab_focus()
+            if links_parms.first_in:
+                links_parms.first_in = False
+                config.get_tree_expanded_collapsed_string(dad)
+                config.set_tree_expanded_collapsed_string(dad, treeview=treeview_2)
+            if not links_parms.sel_iter: links_parms.sel_iter = dad.treestore.get_iter_first()
+            sel_path = dad.treestore.get_path(links_parms.sel_iter)
+            treeview_2.expand_to_path(sel_path)
+            treeview_2.set_cursor(sel_path)
+            treeview_2.scroll_to_cell(sel_path)
+        elif dad.link_type == cons.LINK_TYPE_FILE: entry_file.grab_focus()
+        else: entry_folder.grab_focus()
     def on_radiobutton_link_website_toggled(radiobutton):
         if radiobutton.get_active(): dad.link_type = cons.LINK_TYPE_WEBS
         link_type_changed_on_dialog()
@@ -1012,7 +1050,7 @@ def dialog_link_handle(dad, title, sel_tree_iter):
         if not filepath: return
         dad.pick_dir = os.path.dirname(filepath)
         if dad.links_relative:
-            try: filepath = os.path.relpath(filepath, os.getcwd())
+            try: filepath = os.path.relpath(filepath, dad.file_dir)
             except: print "cannot set relative path for different drives"
         entry_file.set_text(filepath)
     def on_button_browse_for_folder_to_link_to_clicked(self, *args):
@@ -1020,7 +1058,7 @@ def dialog_link_handle(dad, title, sel_tree_iter):
         if not filepath: return
         dad.pick_dir = filepath
         if dad.links_relative:
-            try: filepath = os.path.relpath(filepath, os.getcwd())
+            try: filepath = os.path.relpath(filepath, dad.file_dir)
             except: print "cannot set relative path for different drives"
         entry_folder.set_text(filepath)
     def on_browse_anchors_button_clicked(*args):
@@ -1059,11 +1097,26 @@ def dialog_link_handle(dad, title, sel_tree_iter):
                     treeview.collapse_row(treestore.get_path(links_parms.sel_iter))
                 else:
                     treeview.expand_row(treestore.get_path(links_parms.sel_iter), open_all=False)
+        elif event.type == gtk.gdk.KEY_PRESS:
+            if links_parms.sel_iter:
+                treestore = treeview.get_model()
+                keyname = gtk.gdk.keyval_name(event.keyval)
+                if keyname == cons.STR_KEY_LEFT:
+                    treeview.collapse_row(treestore.get_path(links_parms.sel_iter))
+                elif keyname == cons.STR_KEY_RIGHT:
+                    treeview.expand_row(treestore.get_path(links_parms.sel_iter), open_all=False)
     def on_key_press_links_handle_dialog(widget, event):
-        if gtk.gdk.keyval_name(event.keyval) == cons.STR_RETURN:
+        keyname = gtk.gdk.keyval_name(event.keyval)
+        if keyname == cons.STR_KEY_RETURN:
             try: dialog.get_widget_for_response(gtk.RESPONSE_ACCEPT).clicked()
             except: print cons.STR_PYGTK_222_REQUIRED
             return True
+        elif keyname == cons.STR_KEY_TAB:
+            if dad.link_type == cons.LINK_TYPE_WEBS: radiobutton_file.set_active(True)
+            elif dad.link_type == cons.LINK_TYPE_FILE: radiobutton_folder.set_active(True)
+            elif dad.link_type == cons.LINK_TYPE_FOLD: radiobutton_node.set_active(True)
+            else: radiobutton_webs.set_active(True)
+            return True
         return False
     radiobutton_webs.connect("toggled", on_radiobutton_link_website_toggled)
     radiobutton_node.connect("toggled", on_radiobutton_link_node_anchor_toggled)
@@ -1074,9 +1127,9 @@ def dialog_link_handle(dad, title, sel_tree_iter):
     button_browse_anchor.connect('clicked', on_browse_anchors_button_clicked)
     treeview_2.connect('event-after', on_treeview_event_after)
     dialog.connect("key_press_event", on_key_press_links_handle_dialog)
-    
-    link_type_changed_on_dialog()
+
     content_area.show_all()
+    link_type_changed_on_dialog()
     response = dialog.run()
     dialog.hide()
     if response != gtk.RESPONSE_ACCEPT: return False
@@ -1102,6 +1155,7 @@ def dialog_choose_node(dad, title, treestore, sel_tree_iter):
     dialog.set_default_size(600, 500)
     treeview_2 = gtk.TreeView(treestore)
     treeview_2.set_headers_visible(False)
+    treeview_2.set_search_column(1)
     renderer_pixbuf_2 = gtk.CellRendererPixbuf()
     renderer_text_2 = gtk.CellRendererText()
     column_2 = gtk.TreeViewColumn()
@@ -1114,17 +1168,10 @@ def dialog_choose_node(dad, title, treestore, sel_tree_iter):
     scrolledwindow = gtk.ScrolledWindow()
     scrolledwindow.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
     scrolledwindow.add(treeview_2)
-    config.get_tree_expanded_collapsed_string(dad)
-    config.set_tree_expanded_collapsed_string(dad, treeview=treeview_2)
-    if node_parms.sel_iter:
-        sel_path = treestore.get_path(node_parms.sel_iter)
-        treeview_2.expand_to_path(sel_path)
-        treeview_2.set_cursor(sel_path)
-        treeview_2.scroll_to_cell(sel_path)
     content_area = dialog.get_content_area()
     content_area.pack_start(scrolledwindow)
     def on_key_press_choose_node_dialog(widget, event):
-        if gtk.gdk.keyval_name(event.keyval) == cons.STR_RETURN:
+        if gtk.gdk.keyval_name(event.keyval) == cons.STR_KEY_RETURN:
             try: dialog.get_widget_for_response(gtk.RESPONSE_ACCEPT).clicked()
             except: print cons.STR_PYGTK_222_REQUIRED
             return True
@@ -1146,9 +1193,24 @@ def dialog_choose_node(dad, title, treestore, sel_tree_iter):
                     treeview.collapse_row(treestore.get_path(node_parms.sel_iter))
                 else:
                     treeview.expand_row(treestore.get_path(node_parms.sel_iter), open_all=False)
+        elif event.type == gtk.gdk.KEY_PRESS:
+            if node_parms.sel_iter:
+                treestore = treeview.get_model()
+                keyname = gtk.gdk.keyval_name(event.keyval)
+                if keyname == cons.STR_KEY_LEFT:
+                    treeview.collapse_row(treestore.get_path(node_parms.sel_iter))
+                elif keyname == cons.STR_KEY_RIGHT:
+                    treeview.expand_row(treestore.get_path(node_parms.sel_iter), open_all=False)
     dialog.connect("key_press_event", on_key_press_choose_node_dialog)
     treeview_2.connect('event-after', on_treeview_event_after)
     content_area.show_all()
+    config.get_tree_expanded_collapsed_string(dad)
+    config.set_tree_expanded_collapsed_string(dad, treeview=treeview_2)
+    if node_parms.sel_iter:
+        sel_path = treestore.get_path(node_parms.sel_iter)
+        treeview_2.expand_to_path(sel_path)
+        treeview_2.set_cursor(sel_path)
+        treeview_2.scroll_to_cell(sel_path)
     response = dialog.run()
     dialog.hide()
     return None if response != gtk.RESPONSE_ACCEPT else node_parms.sel_iter
@@ -1190,7 +1252,7 @@ def dialog_selnode_selnodeandsub_alltree(dad, also_selection, also_include_node_
         checkbutton_new_node_page.set_active(dad.last_new_node_page)
         content_area.pack_start(checkbutton_new_node_page)
     def on_key_press_enter_dialog(widget, event):
-        if gtk.gdk.keyval_name(event.keyval) == cons.STR_RETURN:
+        if gtk.gdk.keyval_name(event.keyval) == cons.STR_KEY_RETURN:
             try: dialog.get_widget_for_response(gtk.RESPONSE_ACCEPT).clicked()
             except: print cons.STR_PYGTK_222_REQUIRED
             return True
@@ -1349,7 +1411,7 @@ def bookmarks_handle(dad):
     treeviewselection = treeview.get_selection()
     def on_key_press_liststore(widget, event):
         keyname = gtk.gdk.keyval_name(event.keyval)
-        if keyname == "Delete":
+        if keyname == cons.STR_KEY_DELETE:
             model, tree_iter = treeviewselection.get_selected()
             if tree_iter: model.remove(tree_iter)
     def on_mouse_button_clicked_liststore(widget, event):
